@@ -1,6 +1,6 @@
 package Dist::Zilla::Plugin::ChangelogFromGit;
 {
-  $Dist::Zilla::Plugin::ChangelogFromGit::VERSION = '0.005';
+  $Dist::Zilla::Plugin::ChangelogFromGit::VERSION = '0.015';
 }
 
 # Indent style:
@@ -20,6 +20,7 @@ use DateTime::Infinite;
 use Software::Release;
 use Software::Release::Change;
 use Git::Repository::Log::Iterator;
+use IPC::Cmd qw/run/;
 
 has max_age => (
 	is      => 'ro',
@@ -85,6 +86,16 @@ has earliest_date => (
 	},
 );
 
+has include_message => (
+    is  => 'ro',
+    isa => 'Str',
+);
+
+has exclude_message => (
+    is  => 'ro',
+    isa => 'Str',
+);
+
 sub gather_files {
 	my ($self, $arg) = @_;
 
@@ -92,7 +103,8 @@ sub gather_files {
 
 	my $earliest_date = $self->earliest_date();
 
-	chomp(my @tags = `git tag`);
+	my @tags = $self->rungit([qw/git tag/]);
+	my @head_version_per_tag = ();
 
 	{
 		my $tag_pattern = $self->tag_regexp();
@@ -104,8 +116,14 @@ sub gather_files {
 				next;
 			}
 
-			my $commit = `git show $tags[$i] --pretty='tformat:(((((%ct)))))' | grep '(((((' | head -1`;
-			die $commit unless $commit =~ /\(\(\(\(\((\d+?)\)\)\)\)\)/;
+			my $commit = '';
+			foreach ($self->rungit(['git', 'show', "refs/tags/$tags[$i]", "--pretty='tformat:(((((%ct)))))"])) {
+				next if (! /\(\(\(\(\(/);
+				$commit = $_;
+				last;
+			}
+			die "Failed to find our pretty print format ((((( for tag $tags[$i]: $commit" unless $commit =~ /\(\(\(\(\((\d+?)\)\)\)\)\)/;
+			push(@head_version_per_tag, ($self->rungit(['git', 'rev-list', "refs/tags/$tags[$i]"]))[0]);
 
 			$self->push_release(
 				Software::Release->new(
@@ -121,10 +139,9 @@ sub gather_files {
 	# releases, up to "HEAD".
 
 	{
-		chomp( my $head_version = `git rev-list HEAD | tail -1` );
-		chomp( my $head_time    = `git show --format=%ct -n1 HEAD | head -1` );
+		my $head_version = ($self->rungit([qw/git rev-list HEAD/]))[0];
 
-		if ( not $self->all_releases or $head_version ne $self->get_release(-1)->version()) {
+		if ( not $self->all_releases or ! grep {$head_version eq $_} @head_version_per_tag) {
 			$self->push_release(
 				Software::Release->new(
 					date    => DateTime->now(),
@@ -160,8 +177,22 @@ sub gather_files {
 
 			warn ">>> $release_range\n" if $self->debug();
 
+			my $exclude_message_re = $self->exclude_message();
+			my $include_message_re = $self->include_message();
+
 			my $iter = Git::Repository::Log::Iterator->new($release_range);
 			while (my $log = $iter->next) {
+				next if (
+					defined $exclude_message_re and
+					$log->message() =~ /$exclude_message_re/o
+				);
+
+				next if (
+					defined $include_message_re and
+					$log->message() !~ /$include_message_re/o
+				);
+
+				#print STDERR "LOG: ".$log->message."\n";
 
 				warn("    ", $log->commit(), " ", $log->committer_localtime, "\n") if (
 					$self->debug()
@@ -375,6 +406,12 @@ sub format_datetime {
 	return $datetime->strftime("%F %T %z");
 }
 
+sub rungit {
+	my ($self, $arrayp) = @_;
+	my $buf;
+	run(command => $arrayp, buffer => \$buf);
+	return split("\n", $buf);
+}
 __PACKAGE__->meta->make_immutable;
 no Moose;
 1;
@@ -387,7 +424,7 @@ Dist::Zilla::Plugin::ChangelogFromGit - Write a Changes file from a project's gi
 
 =head1 VERSION
 
-version 0.005
+version 0.015
 
 =head1 SYNOPSIS
 
@@ -396,7 +433,7 @@ their default values.
 
 	[ChangelogFromGit]
 	max_age     = 365
-	tag_regexp  = ^v\d+_\d+$
+	tag_regexp  = ^v(\d+\.\d+)$
 	file_name   = CHANGES
 	wrap_column = 74
 	debug       = 0
@@ -480,6 +517,37 @@ runtime tracing on STDERR.
 	[ChangelogFromGit]
 	debug = 1
 
+=head2 exclude_message = REGULAR_EXPRESSION
+
+C<exclude_message> sets a regular expression which discards matching
+commit messages.  This provides a way to exclude commit messages such
+as 'forgot to include file X' or 'typo'.  The regular expression is
+case sensitive.
+
+	[ChangelogFromGit]
+	exclude_message = ^(forgot|typo)
+
+C<include_message> can be used to do the opposite: exclude all changes
+except ones that match a regular expression.  Using both at once is
+liable to generate empty change logs.
+
+=head2 include_message = REGULAR_EXPRESSION
+
+C<include_message> does the opposite of C<exclude_message>: it sets a
+regular expression which commit messages must match in order to be
+included in the Changes file.  This means that when making a commit
+with a relevant message, you must include text that matches the
+regular expression pattern to have it included in the Changes file.
+All other commit messages are ignored.
+
+The regular expression is case sensitive.
+
+	[ChangelogFromGit]
+	include_message = ^Major
+
+Using both C<include_message> and C<exclude_message> at the same time
+will most likely result in empty change logs.
+
 =head1 HOW IT WORKS
 
 Dist::Zilla::ChangelogFromGit collects the tags matching C<tag_regexp>
@@ -877,7 +945,7 @@ overridable.
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2010-2011 by Rocco Caputo.
+This software is copyright (c) 2010-2013 by Rocco Caputo.
 
 This is free software; you may redistribute it and/or modify it under
 the same terms as the Perl 5 programming language itself.
